import json

from aws_cdk import (
    Stack,
    aws_s3 as s3,
    aws_iam as iam,
    App,
    Environment,
    aws_events_targets as events_targets,
    aws_sns as sns,
    aws_events as events
)
from constructs import Construct
from aws_clamav_malware_scanner_vpc import ServerlessClamscan


class AwsSecureS3StackJ(Stack):
    def __init__(self, scope: Construct, id: str, use_imported_buckets: bool = False, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)
        self.use_imported_buckets = use_imported_buckets

        bucket_1 = s3.Bucket(self, "infectedbucketname", bucket_name="infectedbucketname-851725238313")
        bucket_2 = s3.Bucket(self, "cleanbucketname", bucket_name="cleanbucketname-851725238313")

        # Logic for creating new buckets
        bucket_1 = s3.Bucket(self, "rBucket1", bucket_name="explicit-bucket-name-1")
        bucket_2 = s3.Bucket(self, "rBucket2", bucket_name="explicit-bucket-name-2")
        bucket_3 = s3.Bucket(self, "rBucket3")

        # # Logic for using imported buckets
        # imported_bucket = s3.Bucket.from_bucket_attributes(self, "bucket_name1",
        #                                                    bucket_name="s3-config-test-124312414124142")

        # Create the ServerlessClamscan construct
        bucket_mapping = {
            "explicit-bucket-name-1": json.dumps({
                "infected_bucket": "infectedbucketname-851725238313",
                "clean_bucket": "cleanbucketname-851725238313"
            }),
            "explicit-bucket-name-2": json.dumps({
                "infected_bucket": "infectedbucketname-851725238313",
                "clean_bucket": "cleanbucketname-851725238313"
            }),
            "s3-config-test-124312414124142": json.dumps({
                "infected_bucket": "infectedbucketname-851725238313",
                "clean_bucket": "cleanbucketname-851725238313"
            }),
        }
        bucket_list = [bucket_1, bucket_2, bucket_3]
        sc = ServerlessClamscan(self,
                                "rClamScan",
                                buckets=bucket_list,
                                file_exclusion_patterns=[],
                                file_extension_allow_list=[],
                                file_extension_deny_list=[],
                                accept_responsibility_for_using_imported_bucket=True,
                                bucket_mapping=bucket_mapping,
                                clean_bucket_name="cleanbucketname-851725238313",
                                infected_bucket_name="infectedbucketname-851725238313"
                                )
        sc.add_source_bucket(bucket_1)
        sc.add_source_bucket(bucket_2)
        sc.add_source_bucket(bucket_3)
        # sc.add_source_bucket(imported_bucket)

        # Add result triggers for output of infected files
        infected_topic = sns.Topic(self, "rInfectedTopic")
        if sc.infected_rule is not None:
            sc.infected_rule.add_target(
                events_targets.SnsTopic(
                    infected_topic,
                    message=events.RuleTargetInput.from_event_path('$.detail.responsePayload.message'),
                )
            )

        # Add result triggers for output of clean files
        clean_topic = sns.Topic(self, "rCleanTopic")
        if sc.clean_rule is not None:
            sc.clean_rule.add_target(
                events_targets.SnsTopic(
                    clean_topic,
                    message=events.RuleTargetInput.from_event_path('$.detail.responsePayload.message'),
                )
            )


env = Environment(account="851725238313", region="ap-southeast-2")
app = App()
AwsSecureS3StackJ(app, "AwsSecureS3StackJ", use_imported_buckets=True, env=env)
app.synth()
